#ifndef NRF24LU1_H_
#define NRF24LU1_H_

#include <stddef.h>

#ifndef bool
#define bool unsigned char
#endif
#ifndef false
#define false 0
#endif
#ifndef true
#define true !false
#endif

/////////////////////////////////////////////////////////////////////////////////
// SPI function requirements
//
// The user must define a function to send one byte of data and also return the
//   resulting byte of data data through the SPI port. The function used here
//   has the function prototype
//
//          unsigned char nrf24lu1_spi_send_read_byte(unsigned char byte);
//
// This function should take the argument unsigned char byte and send it through
//   the SPI port to the 24Lu1.  Then, it should wait until the 24Lu1 has returned
//   its response over SPI.  This received byte should be the return value of the
//   function.
//
// You should also change the include file name below to whatever the name of your
//   SPI include file is.
//////////////////////////////////////////////////////////////////////////////////
#include "spi1.h"
#define nrf24lu1_spi_send_read_byte(byte)	spi1_send_read_byte(byte)

/////////////////////////////////////////////////////////////////////////////////
// Delay function requirements
//
// The user must define a function that delays for the specified number of
//       microseconds. This function needs to be as precise as possible, and the use
//   of a timer module within your microcontroller is highly recommended. The
//   function used here has the prototype
//
//          void nrf24lu1_delay_us(unsigned int microseconds);
//
// You should also change the include file name below to whatever the name of your
//   delay include file is.
//////////////////////////////////////////////////////////////////////////////////
#include "delays.h"
#define nrf24lu1_delay_us(microseconds)		DelayUS(microseconds)

//////////////////////////////////////////////////////////////////////////////////
// IO pin definitions
//
// Below you will find several definitions and includes.  The first is an #include
//   for your microcontroller's include file to allow you to use register names
//   rather than numbers.  The next three are to allow you to control the pins on
//   the 24Lu1 that aren't automatically handled by SPI.  These are CE, CSN, and
//   IRQ.
//
// The general format of these defines is a define for the IO register the pin is
//   attached to.  The second define is a mask for the pin.  For example, say that
//   your CE pin is tied to an IO port with the register name IOPORT1. Also, let's
//   say that the IO port is 8-bits wide, and you have attached the pin to pin 0 of
//   the port.  Then your define would look like this:
//
//       #define nrf24lu1_CE_IOREGISTER         IOPORT1
//   #define nrf24lu1_CE_PINMASK                0x01
//
// If you have defines in your include file for individual IO pins, you could use
//   this define in this file, as well.  Using the previous example, assume that in
//   your microcontroller's include file, pin 0 of IOPORT1 has a define like this
//
//   #define IOPORT1_PIN0       0x01
//
// Then, you could make your defines for the CE pin in this file look like this:
//
//       #define nrf24lu1_CE_IOREGISTER         IOPORT1
//   #define nrf24lu1_CE_PINMASK                IOPORT1_PIN0
//
// You should also change the include file name below to whatever the name of your
//   processor's register definition include file is.
/////////////////////////////////////////////////////////////////////////////////////
#include "lpc214x.h"

//defines for uC pins CE pin is connected to
//This is used so that the routines can send TX payload data and
//      properly initialize the nrf24l01 in TX and RX states.
//Change these definitions (and then recompile) to suit your particular application.
#define NRF24LU1_PROG_IOREGISTER	FIO0PIN
#define NRF24LU1_PROG_PINMASK		0x200000

//defines for uC pins CSN pin is connected to
//This is used so that the routines can send properly operate the SPI interface
// on the nrf24l01.
//Change these definitions (and then recompile) to suit your particular application.
#define NRF24LU1_CSN_IOREGISTER		FIO0PIN
#define NRF24LU1_CSN_PINMASK		0x100000

//defines for uC pins IRQ pin is connected to
//This is used so that the routines can poll for IRQ or create an ISR.
//Change these definitions (and then recompile) to suit your particular application.
#define NRF24LU1_IRQ_IOREGISTER		FIO0PIN
#define NRF24LU1_IRQ_PINMASK		0x8000

//flash memory defines
#define	NRF24LU1_NUM_PAGES	32
#define	NRF24LU1_PAGE_SIZE	512

//defines for flash commands to program the nRF24LU1
#define NRF24LU1_FLASH_CMD_WREN			0x06
#define NRF24LU1_FLASH_CMD_WRDIS		0x04
#define NRF24LU1_FLASH_CMD_RDSR			0x05
#define NRF24LU1_FLASH_CMD_WRSR			0x01
#define NRF24LU1_FLASH_CMD_WREN			0x06
#define NRF24LU1_FLASH_CMD_READ			0x03
#define NRF24LU1_FLASH_CMD_PROGRAM		0x02
#define NRF24LU1_FLASH_CMD_ERASE_PAGE	0x52
#define NRF24LU1_FLASH_CMD_ERASE_ALL	0x62
#define NRF24LU1_FLASH_CMD_RDFPCR		0x89
#define NRF24LU1_FLASH_CMD_RDISIP		0x84
#define NRF24LU1_FLASH_CMD_RDISMB		0x85
#define NRF24LU1_FLASH_CMD_ENDEBUG		0x86

//defines for FSR bits
#define NRF24LU1_FSR_RESERVED	0x01
#define NRF24LU1_FSR_RDISIP		0x02
#define NRF24LU1_FSR_RDISMB		0x04
#define NRF24LU1_FSR_INFEN		0x08
#define NRF24LU1_FSR_RDYN		0x10
#define NRF24LU1_FSR_WEN		0x20
#define NRF24LU1_FSR_STP		0x40
#define NRF24LU1_FSR_DBG		0x80

void          nrf24lu1_flash_cmd_wren( void );
void          nrf24lu1_flash_cmd_wrdis(  );
unsigned char nrf24lu1_flash_cmd_rdsr( void );
void          nrf24lu1_flash_cmd_wrsr(  );
void          nrf24lu1_flash_cmd_read( unsigned short address,
    unsigned char *data, unsigned int data_len );
void          nrf24lu1_flash_cmd_program( unsigned short address,
    unsigned char *data, unsigned int data_len );
void          nrf24lu1_flash_cmd_erase_page( unsigned char page_num );
void          nrf24lu1_flash_cmd_erase_all( void );
void          nrf24lu1_flash_cmd_rdfpcr(  );
void          nrf24lu1_flash_cmd_rdisip(  );
void          nrf24lu1_flash_cmd_rdismb(  );
void          nrf24lu1_flash_cmd_endebug(  );

void          nrf24lu1_wait_for_flash_ready(  );

void          nrf24lu1_clear_prog(  );
void          nrf24lu1_set_prog(  );
bool          nrf24lu1_prog_pin_active(  );
void          nrf24lu1_clear_csn(  );
void          nrf24lu1_set_csn(  );
bool          nrf24lu1_csn_pin_active(  );

void          nrf24lu1_execute_command_no_address( unsigned char instruction,
    unsigned char *data, unsigned int len, bool copydata );
void          nrf24lu1_execute_command_one_byte_address( unsigned char
    instruction, unsigned char address, unsigned char *data,
    unsigned int data_len, bool copydata );
void          nrf24lu1_execute_command_two_byte_address( unsigned char
    instruction, unsigned short address, unsigned char *data,
    unsigned int data_len, bool copydata );
void          nrf24lu1_execute_command( unsigned char instruction,
    unsigned char *address, unsigned int address_len, unsigned char *data,
    unsigned int data_len, bool copydata );
void          nrf24lu1_spi_send_read( unsigned char *data, unsigned int len,
    bool copydata );

#endif				/* NRF24LU1_H_ */
